/**
 * @file      Node entry and main driver
 * @author    Jordan Lavatai, Ken Grimes
 * @version   0.0.1
 * @license   AGPL-3.0
 * @copyright jk software 2017
 * @summary   HTTP(S) Router that uses the first directory in the requested URL
 *            as the route name
 */
const fs = require('fs')
const ws = require('ws')
const path = require('path')
const http = require('http')
const https = require('https')
const getport = require('get-port')
const mime = require('mime')
const opts = require('./opts.js')

const skelPage  = fs.readFileSync('./skel.html', { encoding: 'utf8' }).split('<!--STRAPP_SRC-->')
const clientJS  = fs.readFileSync(opts['client-js'])
const hostJS    = fs.readFileSync(opts['host-js'])
const routes    = {}
const httpsOpts = (opts['no-tls'] ?
		   undefined      :
		   {
		     key: fs.readFileSync(opts['ca-key']),
		     cert: fs.readFileSync(opts['ca-cert'])
		   })

const routeConnection = (request,response) => {
  const serveFile = (fPath) => {
    fs.readFile(fPath, { encoding: 'utf8' }, (err, data) => {
      if (err || data == undefined) {
	response.writeHead(404)
	response.end()
      }
      else {
	response.writeHead(200, { 'Content-Type': mime.lookup(fPath) })
	response.write(data)
	response.end()
      }
    })
  }
  const htArgv = request.url.slice(1).split("?")
  let routePath = htArgv[0].split('/')
  let routeName = routePath[0]
  if (routeName === '' || routeName === 'index.html')
    serveFile(opts['index'])
  else if (routeName in opts['bindings']) {
    let localPath = path.normalize(opts['bindings'][routeName].concat(path.sep + routePath.slice(1).join(path.sep)))
    if (localPath.includes(opts['bindings'][routeName])) {
      fs.readdir(localPath, (err, files) => {
	if (err)
	  serveFile(localPath)
	else
	  serveFile(`${localPath}/index.html`)
      })
    }
    else {
      console.log(`SEC: ${localPath} references files not in route`)
    }
  }
  else if (routeName in routes) {
    const route = routes[routeName]
    response.writeHead(200, { 'Content-Type': 'text/html' })
    response.write(`${skelPage[0]}${clientJS}${skelPage[1]}`)
    response.end()
    route.socket.send(request.headers['x-forwarded-for'] || request.connection.remoteAddress)
  }
  else {
    routes[htArgv[0]] = true
    const newRoute = {}
    newRoute.host = request.headers['x-forwarded-for'] || request.connection.remoteAddress
    getport().then( (port) => {
      newRoute.port = port
      if (opts['no-tls'])
	newRoute.httpd = http.createServer()
      else
	newRoute.httpd = https.createServer(httpsOpts)
      newRoute.httpd.listen(newRoute.port)
      newRoute.ws = new ws.Server( { server: newRoute.httpd } )
      newRoute.ws.on('connection', (ws) => { console.log("socket connected"); newRoute.socket = ws; ws.send("CONNECTED") } )
      newRoute.ws.on('message', (msg) => { console.log("Received message" + msg) })
      console.log("Listening for websocket " + newRoute.host + " on port " + newRoute.port)
      routes[htArgv[0]] = newRoute
    }).then(() => {
      response.writeHead(200, { 'Content-Type': 'text/html' })
      response.write(skelPage[0] + 'const _strapp_port = \'' + newRoute.port + '\'\n' + hostJS + skelPage[1])
      response.end()
    })
  }
}

const router = ((opts['no-tls']) ?
		http.createServer(routeConnection) :
		https.createServer(httpsOpts, routeConnection))
      .listen(opts['port'])

//TODO: if ("electron" in process.versions) open a local renderwindow, and route to it

